<?php

namespace Drupal\uikit_image_formatter\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'uikit_lightbox' formatter.
 *
 * @FieldFormatter(
 *   id = "uikit_lightbox",
 *   label = @Translation("Uikit lightbox"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class UikitLightbox extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'content' => [
        'style' => '',
        'lightbox_style' => '',
        'legend' => true,
      ],
      'nb-items' => [
        'default' => 1,
        's' => 2,
        'm' => 3,
        'l' => 4,
        'xl' => 4,
      ],
      'lightbox' => [
        'animation' => 'slide',
        'autoplay' => true,
        'autoplay-interval' => 5000,
        'pause-on-hover' => true,
      ],
      // Implement default settings.
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $image_styles = image_style_options(false);

    $element['content'] = [
      '#title' => $this->t('Content display'),
      '#type' => 'details',
      '#open' => true,
      '#default_value' => [
        $this->getSetting('content')['style'],
        $this->getSetting('content')['lightbox_style'],
        $this->getSetting('content')['legend'],
      ],
    ];

    $element['content']['style'] = [
      '#title' => t('Image style for content'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('content')['style'],
      '#empty_option' => t('None (original image)'),
      '#options' => $image_styles,
    ];

    $element['content']['lightbox_style'] = [
      '#title' => t('Image style for lightbox'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('content')['lightbox_style'],
      '#empty_option' => t('None (original image)'),
      '#options' => $image_styles,
    ];

    $element['content']['legend'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display alt text as caption in content'),
      '#default_value' => $this->getSetting('content')['legend'],
    ];

      // Number element
    $element['nb-items'] = array(
      '#title' => $this->t('Number element '),
      '#type' => 'details',
      '#open' => true,
      '#default_value' => [
        $this->getSetting('nb-items')['default'],
        $this->getSetting('nb-items')['s'],
        $this->getSetting('nb-items')['m'],
        $this->getSetting('nb-items')['l'],
        $this->getSetting('nb-items')['xl'],
      ],
    );

    $element_number = [
      '0' => 'Not use',
      '1' => 1,
      '2' => 2,
      '3' => 3,
      '4' => 4,
      '5' => 5,
      '6' => 6,
    ];

    $element['nb-items']['default'] = [
      '#title' => t('Number element per default'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('nb-items')['default'],
      '#options' => $element_number,
    ];

    $element['nb-items']['s'] = [
      '#title' => t('Number element for small screen (640px and larger)'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('nb-items')['s'],
      '#options' => $element_number,
    ];

    $element['nb-items']['m'] = [
      '#title' => t('Number element for medium screen (960px and larger)'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('nb-items')['m'],
      '#options' => $element_number,
    ];

    $element['nb-items']['l'] = [
      '#title' => t('Number element for large screen (1200px and larger)'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('nb-items')['l'],
      '#options' => $element_number,
    ];

    $element['nb-items']['xl'] = [
      '#title' => t('Number element for extralarge screen (1600px and larger)'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('nb-items')['xl'],
      '#options' => $element_number,
    ];

    $element['lightbox'] = [
      '#title' => $this->t('Lightbox display'),
      '#type' => 'details',
      '#open' => true,
      '#default_value' => [
        $this->getSetting('content')['style'],
        $this->getSetting('content')['legend'],
      ],
    ];

    $animation_effects = [
      'slide' => $this->t('slide'),
      'fade' => $this->t('fade'),
      'scale' => $this->t('scale'),
    ];

    $element['lightbox']['animation'] = [
      '#title' => t('Lightbox animation mode'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('lightbox')['animation'],
      '#options' => $animation_effects,
    ];

    $element['lightbox']['autoplay'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Lightbox autoplays'),
      '#default_value' => $this->getSetting('lightbox')['autoplay'],
    ];

    $element['lightbox']['autoplay-interval'] = [
      '#type' => 'number',
      '#title' => $this->t('The delay between switching slides in autoplay mode.'),
      '#default_value' => $this->getSetting('lightbox')['autoplay-interval'],
      '#min' => 3000,
      '#step' => 1000,
    ];

    $element['lightbox']['pause-on-hover'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Pause autoplay mode on hover.'),
      '#default_value' => $this->getSetting('lightbox')['pause-on-hover'],
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {

    $summary = [];
    // Implement settings summary.
    $image_styles = image_style_options(FALSE);
    // Content Image style
    if (isset($image_styles[$this->getSetting('content')['style']])) {
      $summary[] = $this->t('Content image style: @style', ['@style' => $image_styles[$this->getSetting('content')['style']]]);
    }
    else {
      $summary[] = $this->t('Content image style: Original image');
    }
    // Display legend in content
    $summary[] = $this->t('Display alt text as caption in content: @display_legend', ['@display_legend' => $this->getSetting('content')['legend'] ? 'yes' : 'no']);
    // Image style in lightbox
    if (isset($image_styles[$this->getSetting('lightbox_style')])) {
      $summary[] = $this->t('Lightbox image style: @style', ['@style' => $image_styles[$this->getSetting('lightbox')['style']]]);
    }
    else {
      $summary[] = $this->t('Lightbox image style: Original image');
    }
    // Lightbox animation effect
    $summary[] = $this->t('Lightbox animation mode: @animation', ['@animation' => $this->getSetting('lightbox')['animation']]);
    // Lightbox autoplay
    $summary[] = $this->t('Lightbox autoplays: @autoplay', ['@autoplay' => $this->getSetting('lightbox')['autoplay'] ? 'yes' : 'no']);
    // Lightbox Autoplay interval
    $summary[] = $this->t('Autoplay interval: @autoplay-interval (in milliseconds)', ['@autoplay-interval' => $this->getSetting('lightbox')['autoplay-interval']]);
     // Lightbox Pause on hover
    $summary[] = $this->t('Pause on hover: @pause-on-hover', ['@pause-on-hover' => $this->getSetting('lightbox')['pause-on-hover'] ? 'yes' : 'no']);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode)
  {
    $elements = [];
    $settings = $this->getSettings();

    foreach ($items as $delta => $item) {
      $elements[$delta] = array(
        '#theme' => 'uikit_lightbox',
        '#item' => $item,
        '#display_settings' => $settings,
      );
    }
    return $elements;
  }

}
