<?php

namespace Drupal\uikit_image_formatter\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'uikit_slideshow' formatter.
 *
 * @FieldFormatter(
 *   id = "uikit_slideshow",
 *   label = @Translation("Uikit slideshow"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class UikitSlideshow extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'content' => [
        'style' => '',
        'legend' => TRUE,
      ],
      'slideshow' => [
        'animation' => 'slide',
        'autoplay' => true,
        'autoplay-interval' => '6000',
        'pause-on-hover' => true,
        'finite' => false,
        'min-height' => '',
        'max-height' => '',
        'ratio' => '16:9',
      ],
      'navigation' => [
        'slidenav' => true,
        'dotnav' => false,
        'thumbnav' => false,
        'thumbnav-style' => 'thumbnail',
        'overlay' => true,
      ],
      // Implement default settings.
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $image_styles = image_style_options(TRUE);

    $element['content'] = [
      '#title' => $this->t('Content display'),
      '#type' => 'details',
      '#open' => true,
      '#default_value' => [
        $this->getSetting('content')['style'],
        $this->getSetting('content')['legend'],
      ],
    ];

    $element['content']['style'] = [
      '#title' => t('Image style for content'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('content')['style'],
      '#empty_option' => t('None (original image)'),
      '#options' => $image_styles,
    ];

    $element['content']['legend'] = [
      '#title' => $this->t('Display legend in overlay'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('content')['legend'],
    ];

    $element['slideshow'] = [
      '#title' => $this->t('Slideshow settings'),
      '#type' => 'details',
      '#open' => true,
      '#default_value' => [
        $this->getSetting('slideshow')['animation'],
        $this->getSetting('slideshow')['autoplay'],
        $this->getSetting('slideshow')['autoplay-interval'],
        $this->getSetting('slideshow')['pause-on-hover'],
        $this->getSetting('slideshow')['finite'],
        $this->getSetting('slideshow')['min-height'],
        $this->getSetting('slideshow')['max-height'],
        $this->getSetting('slideshow')['ratio'],
      ],
    ];

    $animation_effects = [
      'slide' => $this->t('slide'),
      'fade' => $this->t('fade'),
      'scale' => $this->t('scale'),
      'pull' => $this->t('pull'),
      'push' => $this->t('push'),
    ];
    $element['slideshow']['animation'] = [
      '#title' => t('Slideshow animation mode'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('slideshow')['animation'],
      '#options' => $animation_effects,
    ];

    $element['slideshow']['autoplay'] = [
      '#title' => $this->t('Slideshow autoplays'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('slideshow')['autoplay'],
    ];

    $element['slideshow']['autoplay-interval'] = [
      '#title' => $this->t('The delay between switching slides in autoplay mode'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('slideshow')['autoplay-interval'],
      '#min' => 3000,
      '#step' => 1000,
    ];

    $element['slideshow']['pause-on-hover'] = [
      '#title' => $this->t('Pause autoplay mode on hover'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('slideshow')['pause-on-hover'],
    ];

    $element['slideshow']['finite'] = [
      '#title' => $this->t('Disable infinite sliding'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('slideshow')['finite'],
    ];

    $element['slideshow']['min-height'] = [
      '#title' => $this->t('A minimum height for slideshow (px)'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('slideshow')['min-height'],
    ];

    $element['slideshow']['max-height'] = [
      '#title' => $this->t('A maximum height for slideshow (px)'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('slideshow')['max-height'],
    ];

    $element['slideshow']['ratio'] = [
      '#title' => $this->t('Image ratio in slideshow'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('slideshow')['ratio'],
      '#description' => $this->t('With "The slideshow always takes up the full width of its parent container. The height adapts according to the defined ratio. To change the default ratio of 16:9, just add the ratio option to the attribute. It\'s recommended to use the same ratio as the background images. For example, just use their width and height, like 1600:1200.'),
    ];

    // Navigation and Overlay
    $element['navigation'] = array(
      '#title' => $this->t('Navigation'),
      '#type' => 'details',
      '#open' => true,
      '#default_value' => [
        $this->getSetting('navigation')['slidenav'],
        $this->getSetting('navigation')['dotnav'],
        $this->getSetting('navigation')['thumbnav'],
        $this->getSetting('navigation')['thumbnav-style'],
      ],
    );

    $element['navigation']['slidenav'] = [
      '#title' => $this->t('Slidenav'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('navigation')['slidenav'],
    ];

    $element['navigation']['dotnav'] = [
      '#title' => $this->t('Dotnav'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('navigation')['dotnav'],
    ];

    $element['navigation']['thumbnav'] = [
      '#title' => $this->t('Thumbnav'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('navigation')['thumbnav'],
    ];

    $element['navigation']['thumbnav-style'] = [
      '#title' => t('Image style for thumbnav'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('navigation')['thumbnav-style'],
      '#empty_option' => t('None (original image)'),
      '#options' => $image_styles,
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    // Implement settings summary.
    $summary = [];
    // Image style in slideshow
    $image_styles = image_style_options(FALSE);
    if (isset($image_styles[$this->getSetting('content')['style']])) {
      $summary[] = $this->t('Content image style: @style', ['@style' => $image_styles[$this->getSetting('content')['style']]]);
    }
    else {
      $summary[] = $this->t('Content image style: Original image');
    }
    // Display legend
    if($this->getSetting('content')['legend']) {
       $summary[] = $this->t('Display legend in overlay: yes');
    }
    // Slideshow animation
    $summary[] = $this->t('Slideshow animation mode: @animation', ['@animation' => $this->getSetting('slideshow')['animation']]);
    // Slideshow autoplay
    $summary[] = $this->t('Slideshow autoplays: @autoplay', ['@autoplay' => $this->getSetting('slideshow')['autoplay'] ? 'yes' : 'no']);
    // Slideshow autoplay interval
    $summary[] = $this->t('Autoplay interval: @autoplay-interval (in milliseconds)', ['@autoplay-interval' => $this->getSetting('slideshow')['autoplay-interval']]);
    // Slideshow pause on hover
    $summary[] = $this->t('Pause on hover: @pause-on-hover', ['@pause-on-hover' => $this->getSetting('slideshow')['pause-on-hover'] ? 'yes' : 'no']);
    // Slideshow infinite loop
    $summary[] = $this->t('Disable infinite sliding: @finite', ['@finite' => $this->getSetting('slideshow')['finite'] ? 'yes' : 'no']);
    // Slideshow min height
    if (!empty($this->getSetting('min-height'))) {
      $summary[] = $this->t('Min height slideshow: @min-height', ['@min-height' => $this->getSetting('slideshow')['min-height']]);
    }
    // Slideshow max height
    if (!empty($this->getSetting('max-height'))) {
      $summary[] = $this->t('Max height slideshow: @max-height', ['@max-height' => $this->getSetting('slideshow')['max-height']]);
    }
    // Slideshow ratio
    $summary[] = $this->t('Ratio: @ratio', ['@ratio' => $this->getSetting('slideshow')['ratio']]);
    // Slidenav
    if($this->getSetting('navigation')['slidenav']) {
      $summary[] = $this->t('Slidenav: yes');
    }
    // Dotnav
    if($this->getSetting('navigation')['dotnav']){
      $summary[] = $this->t('Dotnav: yes');
    }
    // Thumbnav
    if($this->getSetting('navigation')['thumbnav']){
       $summary[] = $this->t('Thumbnav: yes');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $settings = $this->getSettings();

    foreach ($items as $delta => $item) {
      $elements[$delta] = array(
        '#theme' => 'uikit_slideshow',
        '#item' => $item,
        '#display_settings' => $settings,
      );
    }

    return $elements;
  }

}
